==> atmsupport-1.4.tar.gz: atmsupport-1.4/ats_sscfu/kern.c <==
/*
 * Copyright (c) 2003-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Begemot: atmsupport/ats_sscfu/kern.c,v 1.3 2004/12/20 10:44:52 brandt Exp $
 */
#ifdef HAVE_NETGRAPH_H
#include <sys/types.h>
#include <sys/queue.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <limits.h>
#include <err.h>

#include <netnatm/unimsg.h>
#include <netnatm/saal/sscopdef.h>
#include <netnatm/saal/sscfudef.h>

#include <netgraph.h>
#include <netgraph/atm/ng_sscop.h>
#include <netgraph/atm/ng_sscfu.h>

#include "ats.h"
#include "ats_sscfu.h"
#include "testlib.h"

/* debugging flags */
static uint32_t debug_flags;

/* netgraph sockets */
static int dsock, csock = -1;

/*
 *                [     ats_sscop    ]
 *           manage|     |upper  |lower
 *	           |     |upper  |
 *		   |   [sscfu]   |
 *		   |     |lower  |
 *           manage|	 |upper  |
 *                 [sscop]       |
 *               lower|          |
 *                    +----------+
 */
static struct stack stack[] = {
	{ STACK_MKPEER,  ".",      "sscfu",  "upper",  "upper",  "sscfu" },
	{ STACK_MKPEER,  "sscfu:", "sscop",  "lower",  "upper",  "sscop" },
	{ STACK_CONNECT, ".",      "sscop:", "manage", "manage", NULL },
	{ STACK_CONNECT, ".",      "sscop:", "lower",  "lower",  NULL },
	{ STACK_END,     NULL,     NULL,     NULL,     NULL,     NULL }
};

static void dump_cmsg(struct ng_mesg *m, const char *path);

/*
 * Set debugging
 */
static void
kern_debug(u_int flags)
{
	debug_flags = flags;
}

/*
 * Reset the stack
 */
static void
kern_reset(void)
{

	if (NgSendAsciiMsg(csock, ".:manage", "disable") == -1)
		err(1, "disable stack");
	if (NgSendAsciiMsg(csock, ".:upper", "disable") == -1)
		err(1, "disable stack");
}

/*
 * Start the stack
 */
static void
kern_start(void)
{

	if (NgSendMsg(csock, ".:manage", NGM_SSCOP_COOKIE,
	    NGM_SSCOP_SETDEBUG, &debug_flags, sizeof(debug_flags)) == -1)
		err(1, "sscop_setdebug");

	if (NgSendAsciiMsg(csock, ".:upper", "enable") == -1)
		err(1, "enable stack");
	if (NgSendAsciiMsg(csock, ".:manage", "enable") == -1)
		err(1, "enable stack");
}

/*
 * Input at the data socket
 */
static void
ready_dmsg(int fd __unused, void *uap __unused)
{
	char hook[NG_HOOKLEN + 1];
	u_char buf[65536];
	ssize_t ret;
	struct winput *ip;

	ret = NgRecvData(dsock, buf, sizeof(buf), hook);
	if (ret == -1)
		err(1, "NgRecvData");

	if (strcmp(hook, "manage") == 0) {
		struct sscop_marg *marg;
		struct sscop_merr *merr;

		if ((size_t)ret < sizeof(*marg))
			errx(1, "management input too short %zd", ret);
		marg = (struct sscop_marg *)buf;

		if ((ip = malloc(sizeof(*ip))) == NULL)
			err(1, NULL);

		ip->sig = marg->sig;
		if (marg->sig == SSCOP_MERROR_indication) {
			if ((size_t)ret != sizeof(*merr))
				errx(1, "error size wrong %zd", ret);
			merr = (struct sscop_merr *)buf;
			ip->arg1 = merr->err;
			ip->arg2 = merr->cnt;
			ip->msg = NULL;
		} else  {
			ip->msg = uni_msg_alloc(ret - sizeof(*marg));
			if (ip->msg == NULL)
				err(1, NULL);

			memcpy(ip->msg->b_wptr, marg->data,
			    ret - sizeof(*marg));
			ip->msg->b_wptr += ret - sizeof(*marg);
		}
		TAILQ_INSERT_TAIL(&manage_input_list, ip, link);

	} else if (strcmp(hook, "upper") == 0) {
		struct sscfu_arg *arg;

		if ((size_t)ret < sizeof(*arg))
			errx(1, "upper layer input too short %zd", ret);
		arg = (struct sscfu_arg *)buf;

		if ((ip = malloc(sizeof(*ip))) == NULL)
			err(1, NULL);

		if ((ip->msg = uni_msg_alloc(ret - sizeof(*arg))) == NULL)
			err(1, NULL);

		memcpy(ip->msg->b_wptr, arg->data, ret - sizeof(*arg));
		ip->msg->b_wptr += ret - sizeof(*arg);
		ip->sig = arg->sig;

		TAILQ_INSERT_TAIL(&upper_input_list, ip, link);

	} else if (strcmp(hook, "lower") == 0) {
		if ((ip = malloc(sizeof(*ip))) == NULL)
			err(1, NULL);
		if ((ip->msg = uni_msg_alloc(ret)) == NULL)
			err(1, NULL);

		memcpy(ip->msg->b_wptr, buf, ret);
		ip->msg->b_wptr += ret;

		TAILQ_INSERT_TAIL(&lower_input_list, ip, link);

	} else
		errx(1, "input from unknown hook '%s'", hook);
}

static void
kern_lower_to_sscop(struct uni_msg *msg)
{
	int ret;

	ret = NgSendData(dsock, "lower", msg->b_rptr, uni_msg_len(msg));
	if (ret == -1)
		err(1, "NgSendData(lower)");

	uni_msg_destroy(msg);
}

static void
kern_upper_to_sscfu(u_int sig, struct uni_msg *msg)
{
	struct sscfu_arg *a;
	int ret;

	if (msg != NULL) {
		if (uni_msg_prepend(msg, sizeof(*a)))
			err(1, NULL);
	} else {
		if ((msg = uni_msg_alloc(sizeof(*a))) == NULL)
			err(1, NULL);
		msg->b_wptr += sizeof(*a);
	}
	a = (struct sscfu_arg *)msg->b_rptr;
	a->sig = sig;

	ret = NgSendData(dsock, "upper", msg->b_rptr, uni_msg_len(msg));
	if (ret == -1)
		err(1, "NgSendData(upper)");

	uni_msg_destroy(msg);
}

static void
kern_manage_to_sscop(u_int sig, u_int arg1 __unused, u_int arg2 __unused,
    struct uni_msg *msg)
{
	struct sscop_marg *a;
	int ret;

	if (sig != SSCOP_MDATA_request)
		err_file("bad management signal %u", sig);

	if (msg != NULL) {
		if (uni_msg_prepend(msg, sizeof(*a)))
			err(1, NULL);
	} else {
		if ((msg = uni_msg_alloc(sizeof(*a))) == NULL)
			err(1, NULL);
		msg->b_wptr += sizeof(*a);
	}
	a = (struct sscop_marg *)msg->b_rptr;
	a->sig = sig;

	ret = NgSendData(dsock, "manage", msg->b_rptr, uni_msg_len(msg));
	if (ret == -1)
		err(1, "NgSendData(manage)");

	uni_msg_destroy(msg);
}

/*
 * Something happend on the control socket. Try to read a message and to
 * dump it.
 */
static void
dump_cmsg(struct ng_mesg *m, const char *path)
{
	struct ng_mesg *a = (struct ng_mesg *)m->data;

	if (NgSendMsg(csock, path, NGM_GENERIC_COOKIE, NGM_BINARY2ASCII,
	    m, sizeof(*m) + m->header.arglen) < 0)
		err(1, "NgSendMsg: BINARY2ASCII");
	if (NgRecvMsg(csock, m, MAXMSG, NULL) < 0)
		err(1, "NgRecvMsg: BINARY2ASCII");

	printf("received %s '%s'(%d) from '%s':\n",
	    (a->header.flags & NGF_RESP) ? "response" : "request",
	    a->header.cmdstr, a->header.cmd, path);

	if (*a->data != '\0')
		printf("Args: '%s'\n", a->data);
}

static void
ready_cmsg(int fd __unused, void *uap __unused)
{
	char path[NG_PATHLEN + 1];
	u_char buf[MAXMSG];
	struct ng_mesg *m = (struct ng_mesg *)buf;

	if (NgRecvMsg(csock, m, sizeof(buf), path) < 0)
		err(1, "NgRecvMsg");
	dump_cmsg(m, path);
}

/*
 * Remove the stack
 */
static void
kill_func(void)
{

	if (csock != -1)
		if (NgSendAsciiMsg(csock, ".:manage", "shutdown") == -1)
			err(1, "killing stack");
}

/*
 * Instantiate a user space stack
 */
static void
kern_init(void)
{

	atexit(kill_func);
	build_stack(&csock, &dsock, stack);

	ats_register_fd(csock, ready_cmsg, NULL);
	ats_register_fd(dsock, ready_dmsg, NULL);
}

const struct sscfu_stack kern_stack = {
	kern_init,
	kern_debug,
	kern_upper_to_sscfu,
	kern_manage_to_sscop,
	kern_lower_to_sscop,
	kern_reset,
	kern_start,
};
#endif

==> atmsupport-1.7.tar.gz: atmsupport-1.7/ats_sscfu/kern.c <==
/*
 * Copyright (c) 2003-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Begemot: atmsupport/ats_sscfu/kern.c,v 1.4 2005/04/22 15:57:58 brandt_h Exp $
 */
#ifdef HAVE_NETGRAPH_H
#include <sys/types.h>
#include <sys/queue.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <limits.h>
#include <err.h>

#include <netnatm/unimsg.h>
#include <netnatm/saal/sscopdef.h>
#include <netnatm/saal/sscfudef.h>

#include <netgraph.h>
#include <netgraph/atm/ng_sscop.h>
#include <netgraph/atm/ng_sscfu.h>

#include "ats.h"
#include "ats_sscfu.h"
#include "testlib.h"

/* debugging flags */
static uint32_t debug_flags;

/* netgraph sockets */
static int dsock, csock = -1;

/*
 *                [     ats_sscop    ]
 *           manage|     |upper  |lower
 *	           |     |upper  |
 *		   |   [sscfu]   |
 *		   |     |lower  |
 *           manage|	 |upper  |
 *                 [sscop]       |
 *               lower|          |
 *                    +----------+
 */
static struct stack stack[] = {
	{ STACK_MKPEER,  ".",      "sscfu",  "upper",  "upper",  "sscfu" },
	{ STACK_MKPEER,  "sscfu:", "sscop",  "lower",  "upper",  "sscop" },
	{ STACK_CONNECT, ".",      "sscop:", "manage", "manage", NULL },
	{ STACK_CONNECT, ".",      "sscop:", "lower",  "lower",  NULL },
	{ STACK_END,     NULL,     NULL,     NULL,     NULL,     NULL }
};

static void dump_cmsg(struct ng_mesg *m, const char *path);

/*
 * Set debugging
 */
static void
kern_debug(u_int flags)
{
	debug_flags = flags;
}

/*
 * Reset the stack
 */
static void
kern_reset(void)
{

	if (NgSendAsciiMsg(csock, ".:manage", "disable") == -1)
		err(1, "disable stack");
	if (NgSendAsciiMsg(csock, ".:upper", "disable") == -1)
		err(1, "disable stack");
}

/*
 * Start the stack
 */
static void
kern_start(void)
{

	if (NgSendMsg(csock, ".:manage", NGM_SSCOP_COOKIE,
	    NGM_SSCOP_SETDEBUG, &debug_flags, sizeof(debug_flags)) == -1)
		err(1, "sscop_setdebug");

	if (NgSendAsciiMsg(csock, ".:upper", "enable") == -1)
		err(1, "enable stack");
	if (NgSendAsciiMsg(csock, ".:manage", "enable") == -1)
		err(1, "enable stack");
}

/*
 * Input at the data socket
 */
static void
ready_dmsg(int fd __unused, void *uap __unused)
{
	char hook[NG_HOOKLEN + 1];
	u_char buf[65536];
	ssize_t ret;
	struct winput *ip;

	ret = NgRecvData(dsock, buf, sizeof(buf), hook);
	if (ret == -1)
		err(1, "NgRecvData");

	if (strcmp(hook, "manage") == 0) {
		struct sscop_marg *marg;
		struct sscop_merr *merr;

		if ((size_t)ret < sizeof(*marg))
			errx(1, "management input too short %zd", ret);
		marg = (struct sscop_marg *)(void *)buf;

		if ((ip = malloc(sizeof(*ip))) == NULL)
			err(1, NULL);

		ip->sig = marg->sig;
		if (marg->sig == SSCOP_MERROR_indication) {
			if ((size_t)ret != sizeof(*merr))
				errx(1, "error size wrong %zd", ret);
			merr = (struct sscop_merr *)(void *)buf;
			ip->arg1 = merr->err;
			ip->arg2 = merr->cnt;
			ip->msg = NULL;
		} else  {
			ip->msg = uni_msg_alloc(ret - sizeof(*marg));
			if (ip->msg == NULL)
				err(1, NULL);

			memcpy(ip->msg->b_wptr, marg->data,
			    ret - sizeof(*marg));
			ip->msg->b_wptr += ret - sizeof(*marg);
		}
		TAILQ_INSERT_TAIL(&manage_input_list, ip, link);

	} else if (strcmp(hook, "upper") == 0) {
		struct sscfu_arg *arg;

		if ((size_t)ret < sizeof(*arg))
			errx(1, "upper layer input too short %zd", ret);
		arg = (struct sscfu_arg *)(void *)buf;

		if ((ip = malloc(sizeof(*ip))) == NULL)
			err(1, NULL);

		if ((ip->msg = uni_msg_alloc(ret - sizeof(*arg))) == NULL)
			err(1, NULL);

		memcpy(ip->msg->b_wptr, arg->data, ret - sizeof(*arg));
		ip->msg->b_wptr += ret - sizeof(*arg);
		ip->sig = arg->sig;

		TAILQ_INSERT_TAIL(&upper_input_list, ip, link);

	} else if (strcmp(hook, "lower") == 0) {
		if ((ip = malloc(sizeof(*ip))) == NULL)
			err(1, NULL);
		if ((ip->msg = uni_msg_alloc(ret)) == NULL)
			err(1, NULL);

		memcpy(ip->msg->b_wptr, buf, ret);
		ip->msg->b_wptr += ret;

		TAILQ_INSERT_TAIL(&lower_input_list, ip, link);

	} else
		errx(1, "input from unknown hook '%s'", hook);
}

static void
kern_lower_to_sscop(struct uni_msg *msg)
{
	int ret;

	ret = NgSendData(dsock, "lower", msg->b_rptr, uni_msg_len(msg));
	if (ret == -1)
		err(1, "NgSendData(lower)");

	uni_msg_destroy(msg);
}

static void
kern_upper_to_sscfu(u_int sig, struct uni_msg *msg)
{
	struct sscfu_arg *a;
	int ret;

	if (msg != NULL) {
		if (uni_msg_prepend(msg, sizeof(*a)))
			err(1, NULL);
	} else {
		if ((msg = uni_msg_alloc(sizeof(*a))) == NULL)
			err(1, NULL);
		msg->b_wptr += sizeof(*a);
	}
	a = (struct sscfu_arg *)(void *)msg->b_rptr;
	a->sig = sig;

	ret = NgSendData(dsock, "upper", msg->b_rptr, uni_msg_len(msg));
	if (ret == -1)
		err(1, "NgSendData(upper)");

	uni_msg_destroy(msg);
}

static void
kern_manage_to_sscop(u_int sig, u_int arg1 __unused, u_int arg2 __unused,
    struct uni_msg *msg)
{
	struct sscop_marg *a;
	int ret;

	if (sig != SSCOP_MDATA_request)
		err_file("bad management signal %u", sig);

	if (msg != NULL) {
		if (uni_msg_prepend(msg, sizeof(*a)))
			err(1, NULL);
	} else {
		if ((msg = uni_msg_alloc(sizeof(*a))) == NULL)
			err(1, NULL);
		msg->b_wptr += sizeof(*a);
	}
	a = (struct sscop_marg *)(void *)msg->b_rptr;
	a->sig = sig;

	ret = NgSendData(dsock, "manage", msg->b_rptr, uni_msg_len(msg));
	if (ret == -1)
		err(1, "NgSendData(manage)");

	uni_msg_destroy(msg);
}

/*
 * Something happend on the control socket. Try to read a message and to
 * dump it.
 */
static void
dump_cmsg(struct ng_mesg *m, const char *path)
{
	struct ng_mesg *a = (struct ng_mesg *)(void *)m->data;

	if (NgSendMsg(csock, path, NGM_GENERIC_COOKIE, NGM_BINARY2ASCII,
	    m, sizeof(*m) + m->header.arglen) < 0)
		err(1, "NgSendMsg: BINARY2ASCII");
	if (NgRecvMsg(csock, m, MAXMSG, NULL) < 0)
		err(1, "NgRecvMsg: BINARY2ASCII");

	printf("received %s '%s'(%d) from '%s':\n",
	    (a->header.flags & NGF_RESP) ? "response" : "request",
	    a->header.cmdstr, a->header.cmd, path);

	if (*a->data != '\0')
		printf("Args: '%s'\n", a->data);
}

static void
ready_cmsg(int fd __unused, void *uap __unused)
{
	char path[NG_PATHLEN + 1];
	u_char buf[MAXMSG];
	struct ng_mesg *m = (struct ng_mesg *)(void *)buf;

	if (NgRecvMsg(csock, m, sizeof(buf), path) < 0)
		err(1, "NgRecvMsg");
	dump_cmsg(m, path);
}

/*
 * Remove the stack
 */
static void
kill_func(void)
{

	if (csock != -1)
		if (NgSendAsciiMsg(csock, ".:manage", "shutdown") == -1)
			err(1, "killing stack");
}

/*
 * Instantiate a user space stack
 */
static void
kern_init(void)
{

	atexit(kill_func);
	build_stack(&csock, &dsock, stack);

	ats_register_fd(csock, ready_cmsg, NULL);
	ats_register_fd(dsock, ready_dmsg, NULL);
}

const struct sscfu_stack kern_stack = {
	kern_init,
	kern_debug,
	kern_upper_to_sscfu,
	kern_manage_to_sscop,
	kern_lower_to_sscop,
	kern_reset,
	kern_start,
};
#endif
